@@ -13,7 +13,7 @@
  * for more details.
  */
 
-#include "owncloudpropagator.h"
+#include "owncloudpropagator_p.h"
 #include "syncjournaldb.h"
 #include "syncjournalfilerecord.h"
 #include "utility.h"
@@ -30,6 +30,7 @@
 #include <QDebug>
 #include <QDateTime>
 #include <qstack.h>
+#include <QCoreApplication>
 
 #include <neon/ne_basic.h>
 #include <neon/ne_socket.h>
@@ -53,25 +54,6 @@ extern "C" void csync_win32_set_file_hidden( const char *file, bool h );
 
 namespace Mirall {
 
-/* Helper for QScopedPointer<>, to be used as the deleter.
- * QScopePointer will call the right overload of cleanup for the pointer it holds
- */
-struct ScopedPointerHelpers {
-    static inline void cleanup(hbf_transfer_t *pointer) { if (pointer) hbf_free_transfer(pointer); }
-    static inline void cleanup(ne_request *pointer) { if (pointer) ne_request_destroy(pointer); }
-    static inline void cleanup(ne_decompress *pointer) { if (pointer) ne_decompress_destroy(pointer); }
-//     static inline void cleanup(ne_propfind_handler *pointer) { if (pointer) ne_propfind_destroy(pointer); }
-};
-
-#define DECLARE_JOB(NAME) \
-class NAME : public PropagateItemJob { \
-  /* Q_OBJECT */ \
-public: \
-    NAME(OwncloudPropagator* propagator,const SyncFileItem& item) \
-    : PropagateItemJob(propagator, item) {} \
-    void start(); \
-};
-
 void PropagateItemJob::done(SyncFileItem::Status status, const QString &errorString)
 {
     _item._errorString = errorString;
@@ -80,7 +62,7 @@ void PropagateItemJob::done(SyncFileItem::Status status, const QString &errorStr
     // Blacklisting
     int retries = 0;
 
-    if( _item._httpErrorCode == 403 || _item._httpErrorCode == 413 || _item._httpErrorCode == 415 ) {
+    if( _item._httpErrorCode == 403 ||_item._httpErrorCode == 413 || _item._httpErrorCode == 415 ) {
         qDebug() << "Fatal Error condition" << _item._httpErrorCode << ", forbid retry!";
         retries = -1;
     } else {
@@ -117,6 +99,84 @@ void PropagateItemJob::done(SyncFileItem::Status status, const QString &errorStr
     emit finished(status);
 }
 
+
+/**
+ * For delete or remove, check that we are not removing from a shared directory.
+ * If we are, try to restore the file
+ *
+ * Return true if the problem is handled.
+ */
+bool PropagateItemJob::checkForProblemsWithShared(const QString& msg)
+{
+    QString errorString = QString::fromUtf8(ne_get_error(_propagator->_session));
+    int httpStatusCode = errorString.mid(0, errorString.indexOf(QChar(' '))).toInt();
+
+    PropagateItemJob *newJob = NULL;
+
+    if( httpStatusCode == 403 && _propagator->isInSharedDirectory(_item._file )) {
+        if( _item._type != SyncFileItem::Directory ) {
+            SyncFileItem downloadItem(_item);
+            if (downloadItem._instruction == CSYNC_INSTRUCTION_NEW) {
+                // don't try to recover pushing new files
+                return false;
+            } else if (downloadItem._instruction == CSYNC_INSTRUCTION_SYNC) {
+                // we modified the file locally, jsut create a conflict then
+                downloadItem._instruction = CSYNC_INSTRUCTION_CONFLICT;
+            } else {
+                // the file was removed or renamed, just recover the old one
+                downloadItem._instruction = CSYNC_INSTRUCTION_SYNC;
+            }
+            downloadItem._dir = SyncFileItem::Down;
+            newJob = new PropagateDownloadFile(_propagator, downloadItem);
+        } else {
+            // Directories are harder to recover.
+            // But just re-create the directory, next sync will be able to recover the files
+            SyncFileItem mkdirItem(_item);
+            mkdirItem._instruction = CSYNC_INSTRUCTION_SYNC;
+            mkdirItem._dir = SyncFileItem::Down;
+            newJob = new PropagateLocalMkdir(_propagator, mkdirItem);
+            // Also remove the inodes and fileid from the db so no further renames are tried for
+            // this item.
+            _propagator->_journal->avoidRenamesOnNextSync(_item._file);
+        }
+        if( newJob )  {
+            newJob->setRestoreJobMsg(msg);
+            _restoreJob.reset(newJob);
+            connect(_restoreJob.data(), SIGNAL(completed(SyncFileItem)),
+                    this, SLOT(slotRestoreJobCompleted(SyncFileItem)));
+            _restoreJob->start();
+        }
+        return true;
+    }
+    return false;
+}
+
+void PropagateItemJob::slotRestoreJobCompleted(const SyncFileItem& item )
+{
+    QString msg;
+    if(_restoreJob) {
+        msg = _restoreJob->restoreJobMsg();
+        _restoreJob->setRestoreJobMsg();
+    }
+
+    if( item._status == SyncFileItem::Success ||  item._status == SyncFileItem::Conflict) {
+        done( SyncFileItem::SoftError, msg);
+    } else {
+        done( item._status, tr("A file or directory was removed from a read only share, but restoring failed: %1").arg(item._errorString) );
+    }
+}
+
+
+QString PropagateItemJob::restoreJobMsg()
+{
+    return _restoreJobMsg;
+}
+
+void PropagateItemJob::setRestoreJobMsg( const QString& msg )
+{
+    _restoreJobMsg = msg;
+}
+
 // compare two files with given filename and return true if they have the same content
 static bool fileEquals(const QString &fn1, const QString &fn2) {
     QFile f1(fn1);
@@ -170,8 +230,6 @@ static bool removeRecursively(const QString &path)
     return success;
 }
 
-DECLARE_JOB(PropagateLocalRemove)
-
 void PropagateLocalRemove::start()
 {
     QString filename = _propagator->_localDir +  _item._file;
@@ -194,8 +252,6 @@ void PropagateLocalRemove::start()
     emit progress(Progress::EndDelete, _item, _item._size, _item._size);
 }
 
-DECLARE_JOB(PropagateLocalMkdir)
-
 void PropagateLocalMkdir::start()
 {
     QDir d;
@@ -206,8 +262,6 @@ void PropagateLocalMkdir::start()
     done(SyncFileItem::Success);
 }
 
-DECLARE_JOB(PropagateRemoteRemove)
-
 void PropagateRemoteRemove::start()
 {
     QScopedPointer<char, QScopedPointerPodDeleter> uri(
@@ -215,18 +269,22 @@ void PropagateRemoteRemove::start()
     emit progress(Progress::StartDelete, _item, 0, _item._size);
     qDebug() << "** DELETE " << uri.data();
     int rc = ne_delete(_propagator->_session, uri.data());
+
+    if( checkForProblemsWithShared(tr("The file has been removed from a read only share. It was restored.")) ) {
+        return;
+    }
+
     /* Ignore the error 404,  it means it is already deleted */
     if (updateErrorFromSession(rc, 0, 404)) {
         return;
     }
+
     _propagator->_journal->deleteFileRecord(_item._originalFile, _item._isDirectory);
     _propagator->_journal->commit("Remote Remove");
     done(SyncFileItem::Success);
     emit progress(Progress::EndDelete, _item, _item._size, _item._size);
 }
 
-DECLARE_JOB(PropagateRemoteMkdir)
-
 void PropagateRemoteMkdir::start()
 {
     QScopedPointer<char, QScopedPointerPodDeleter> uri(
@@ -253,62 +311,8 @@ static QByteArray parseEtag(const char *header) {
     return arr;
 }
 
-class PropagateUploadFile: public PropagateItemJob {
-public:
-    explicit PropagateUploadFile(OwncloudPropagator* propagator,const SyncFileItem& item)
-        : PropagateItemJob(propagator, item) {}
-    void start();
-private:
-    // Log callback for httpbf
-    static void _log_callback(const char *func, const char *text, void*)
-    {
-        qDebug() << "  " << func << text;
-    }
-
-    // abort callback for httpbf
-    static int _user_want_abort(void *userData)
-    {
-        return  static_cast<PropagateUploadFile *>(userData)->_propagator->_abortRequested->fetchAndAddRelaxed(0);
-    }
-
-    // callback from httpbf when a chunk is finished
-    static void chunk_finished_cb(hbf_transfer_s *trans, int chunk, void* userdata)
-    {
-        PropagateUploadFile *that = static_cast<PropagateUploadFile *>(userdata);
-        Q_ASSERT(that);
-        that->_chunked_done += trans->block_arr[chunk]->size;
-        if (trans->block_cnt > 1) {
-            SyncJournalDb::UploadInfo pi;
-            pi._valid = true;
-            pi._chunk = chunk + 1; // next chunk to start with
-            pi._transferid = trans->transfer_id;
-            pi._modtime =  QDateTime::fromTime_t(trans->modtime);
-            that->_propagator->_journal->setUploadInfo(that->_item._file, pi);
-            that->_propagator->_journal->commit("Upload info");
-        }
-    }
-
-    static void notify_status_cb(void* userdata, ne_session_status status,
-                          const ne_session_status_info* info)
-    {
-        PropagateUploadFile* that = reinterpret_cast<PropagateUploadFile*>(userdata);
-
-        if (status == ne_status_sending && info->sr.total > 0) {
-            emit that->progress(Progress::Context, that->_item,
-                                that->_chunked_done + info->sr.progress,
-                                that->_chunked_total_size ? that->_chunked_total_size : info->sr.total );
-
-            that->limitBandwidth(that->_chunked_done + info->sr.progress,  that->_propagator->_uploadLimit);
-        }
-    }
-
-    qint64 _chunked_done; // amount of bytes already sent with the previous chunks
-    qint64 _chunked_total_size; // total size of the whole file
-};
-
 void PropagateUploadFile::start()
 {
-    emit progress(Progress::StartUpload, _item, 0, _item._size);
 
     QFile file(_propagator->_localDir + _item._file);
     if (!file.open(QIODevice::ReadOnly)) {
@@ -336,10 +340,25 @@ void PropagateUploadFile::start()
         hbf_set_abort_callback(trans.data(), _user_want_abort);
         trans.data()->chunk_finished_cb = chunk_finished_cb;
         Q_ASSERT(trans);
+
         state = hbf_splitlist(trans.data(), file.handle());
 
+        // If the source file has changed during upload, it is detected and the
+        // variable _previousFileSize is set accordingly. The propagator waits a
+        // couple of seconds and retries.
+        if(_previousFileSize > 0) {
+            qDebug() << "File size changed underway: " << trans->stat_size - _previousFileSize;
+            // Report the change of the overall transmission size to the propagator
+            _propagator->overallTransmissionSizeChanged(qint64(trans->stat_size - _previousFileSize));
+            // update the item's values to the current from trans. hbf_splitlist does a stat
+            _item._size = trans->stat_size;
+            _item._modtime = trans->modtime;
+
+        }
+        emit progress(Progress::StartUpload, _item, 0, trans->stat_size);
+
         if (progressInfo._valid) {
-            if (progressInfo._modtime.toTime_t() == (uint)_item._modtime) {
+            if (Utility::qDateTimeToTime_t(progressInfo._modtime) == _item._modtime) {
                 trans->start_id = progressInfo._chunk;
                 trans->transfer_id = progressInfo._transferid;
             }
@@ -382,6 +401,11 @@ void PropagateUploadFile::start()
                 if( attempts++ < 5 ) { /* FIXME: How often do we want to try? */
                     qDebug("SOURCE file has changed during upload, retry #%d in %d seconds!", attempts, 2*attempts);
                     sleep(2*attempts);
+                    if( _previousFileSize == 0 ) {
+                        _previousFileSize = _item._size;
+                    } else {
+                        _previousFileSize = trans->stat_size;
+                    }
                     continue;
                 }
 
@@ -392,8 +416,9 @@ void PropagateUploadFile::start()
                 done( SyncFileItem::SoftError, errMsg );
             } else {
                 // Other HBF error conditions.
-                // FIXME: find out the error class.
-                _item._httpErrorCode = hbf_fail_http_code(trans.data());
+                if(checkForProblemsWithShared(tr("The file was edited locally but is part of a read only share. "
+                                                 "It is restored and your edit is in the conflict file.")))
+                    return;
                 done(SyncFileItem::NormalError, hbf_error_string(trans.data(), state));
             }
             return;
@@ -404,7 +429,8 @@ void PropagateUploadFile::start()
         if( trans->modtime_accepted ) {
             _item._etag = parseEtag(hbf_transfer_etag( trans.data() ));
         } else {
-            updateMTimeAndETag(uri.data(), _item._modtime);
+            if (!updateMTimeAndETag(uri.data(), _item._modtime))
+                return;
         }
 
         _propagator->_journal->setFileRecord(SyncJournalFileRecord(_item, _propagator->_localDir + _item._file));
@@ -453,6 +479,38 @@ void PropagateUploadFile::start()
     } while( true );
 }
 
+void PropagateUploadFile::chunk_finished_cb(hbf_transfer_s *trans, int chunk, void* userdata)
+{
+  PropagateUploadFile *that = static_cast<PropagateUploadFile *>(userdata);
+  Q_ASSERT(that);
+  that->_chunked_done += trans->block_arr[chunk]->size;
+  if (trans->block_cnt > 1) {
+    SyncJournalDb::UploadInfo pi;
+    pi._valid = true;
+    pi._chunk = chunk + 1; // next chunk to start with
+    pi._transferid = trans->transfer_id;
+    pi._modtime =  Utility::qDateTimeFromTime_t(trans->modtime);
+    that->_propagator->_journal->setUploadInfo(that->_item._file, pi);
+    that->_propagator->_journal->commit("Upload info");
+  }
+}
+
+void PropagateUploadFile::notify_status_cb(void* userdata, ne_session_status status,
+                             const ne_session_status_info* info)
+{
+  PropagateUploadFile* that = reinterpret_cast<PropagateUploadFile*>(userdata);
+
+  if (status == ne_status_sending && info->sr.total > 0) {
+    emit that->progress(Progress::Context, that->_item,
+                        that->_chunked_done + info->sr.progress,
+                        that->_chunked_total_size ? that->_chunked_total_size : info->sr.total );
+
+     QCoreApplication::processEvents();
+    that->limitBandwidth(that->_chunked_done + info->sr.progress,  that->_propagator->_uploadLimit);
+  }
+}
+
+
 
 static QString parseFileId(ne_request *req) {
     QString fileId;
@@ -464,7 +522,7 @@ static QString parseFileId(ne_request *req) {
     return fileId;
 }
 
-void PropagateItemJob::updateMTimeAndETag(const char* uri, time_t mtime)
+bool PropagateItemJob::updateMTimeAndETag(const char* uri, time_t mtime)
 {
     QByteArray modtime = QByteArray::number(qlonglong(mtime));
     ne_propname pname;
@@ -488,12 +546,8 @@ void PropagateItemJob::updateMTimeAndETag(const char* uri, time_t mtime)
     // get the etag
     QScopedPointer<ne_request, ScopedPointerHelpers> req(ne_request_create(_propagator->_session, "HEAD", uri));
     int neon_stat = ne_request_dispatch(req.data());
-    const ne_status *status = ne_get_status(req.data());
-    if( neon_stat != NE_OK || status->klass != 2 ) {
-        // error happend
-        _item._httpErrorCode = status->code;
-        qDebug() << "Could not issue HEAD request for ETag." << ne_get_error(_propagator->_session);
-        _item._errorString = ne_get_error( _propagator->_session );
+    if (updateErrorFromSession(neon_stat, req.data())) {
+        return false;
     } else {
         _item._etag = parseEtag(ne_get_response_header(req.data(), "etag"));
         QString fid = parseFileId(req.data());
@@ -507,6 +561,7 @@ void PropagateItemJob::updateMTimeAndETag(const char* uri, time_t mtime)
                 qDebug() << "FileID is " << _item._fileId;
             }
         }
+        return true;
     }
 }
 
@@ -538,139 +593,112 @@ void PropagateItemJob::limitBandwidth(qint64 progress, qint64 bandwidth_limit)
     }
 }
 
-class PropagateDownloadFile: public PropagateItemJob {
-public:
-    explicit PropagateDownloadFile(OwncloudPropagator* propagator,const SyncFileItem& item)
-        : PropagateItemJob(propagator, item), _file(0) {}
-    void start();
+int PropagateDownloadFile::content_reader(void *userdata, const char *buf, size_t len)
+{
+    PropagateDownloadFile *that = static_cast<PropagateDownloadFile *>(userdata);
+    size_t written = 0;
 
-private:
-    QFile *_file;
-    QScopedPointer<ne_decompress, ScopedPointerHelpers> _decompress;
-    QString errorString;
-    QByteArray _expectedEtagForResume;
+    if (that->_propagator->_abortRequested->fetchAndAddRelaxed(0)) {
+        ne_set_error(that->_propagator->_session, "%s", tr("Sync was aborted by user.").toUtf8().data());
+        return NE_ERROR;
+    }
 
-    static int content_reader(void *userdata, const char *buf, size_t len)
-    {
-        PropagateDownloadFile *that = static_cast<PropagateDownloadFile *>(userdata);
-        size_t written = 0;
-
-        if (that->_propagator->_abortRequested->fetchAndAddRelaxed(0)) {
-            ne_set_error(that->_propagator->_session, "Aborted by user");
+    if(buf) {
+        written = that->_file->write(buf, len);
+        if( len != written || that->_file->error() != QFile::NoError) {
+            qDebug() << "WRN: content_reader wrote wrong num of bytes:" << len << "," << written;
             return NE_ERROR;
         }
-
-        if(buf) {
-            written = that->_file->write(buf, len);
-            if( len != written || that->_file->error() != QFile::NoError) {
-                qDebug() << "WRN: content_reader wrote wrong num of bytes:" << len << "," << written;
-                return NE_ERROR;
-            }
-            return NE_OK;
-        }
-        return NE_ERROR;
+        return NE_OK;
     }
 
-    static int do_not_accept (void *userdata, ne_request *req, const ne_status *st)
-    {
-        Q_UNUSED(userdata);
-        Q_UNUSED(req);
-        Q_UNUSED(st);
+    return NE_ERROR;
+}
 
-        return 0; // ignore this response
+/*
+ * This hook is called after the response is here from the server, but before
+ * the response body is parsed. It decides if the response is compressed and
+ * if it is it installs the compression reader accordingly.
+ * If the response is not compressed, the normal response body reader is installed.
+ */
+void PropagateDownloadFile::install_content_reader( ne_request *req, void *userdata, const ne_status *status )
+{
+    PropagateDownloadFile *that = static_cast<PropagateDownloadFile *>(userdata);
+
+    Q_UNUSED(status);
+
+    if( !that ) {
+        qDebug("Error: install_content_reader called without valid write context!");
+        return;
     }
 
-    static int do_not_download_content_reader(void *userdata, const char *buf, size_t len)
-    {
-        Q_UNUSED(userdata);
-        Q_UNUSED(buf);
-        Q_UNUSED(len);
-        return NE_ERROR;
-    }
-
-    /*
-    * This hook is called after the response is here from the server, but before
-    * the response body is parsed. It decides if the response is compressed and
-    * if it is it installs the compression reader accordingly.
-    * If the response is not compressed, the normal response body reader is installed.
-    */
-    static void install_content_reader( ne_request *req, void *userdata, const ne_status *status )
-    {
-        PropagateDownloadFile *that = static_cast<PropagateDownloadFile *>(userdata);
-
-        Q_UNUSED(status);
-
-        if( !that ) {
-            qDebug("Error: install_content_reader called without valid write context!");
-            return;
-        }
-
-        if( ne_get_status(req)->klass != 2 ) {
-            qDebug() << "Request class != 2, aborting.";
-            ne_add_response_body_reader( req, do_not_accept,
-                                         do_not_download_content_reader,
-                                         (void*) that );
-            return;
-        }
-
-        QByteArray reason_phrase = ne_get_status(req)->reason_phrase;
-        if(reason_phrase == QByteArray("Connection established")) {
-            ne_add_response_body_reader( req, ne_accept_2xx,
-                                        content_reader,
+    if( ne_get_status(req)->klass != 2 ) {
+        qDebug() << "Request class != 2, aborting.";
+        ne_add_response_body_reader( req, do_not_accept,
+                                        do_not_download_content_reader,
                                         (void*) that );
-            return;
-        }
+        return;
+    }
 
-        QByteArray etag = parseEtag(ne_get_response_header(req, "etag"));
-        if(etag.isEmpty())
-            etag = parseEtag(ne_get_response_header(req, "ETag"));
+    QByteArray reason_phrase = ne_get_status(req)->reason_phrase;
+    if(reason_phrase == QByteArray("Connection established")) {
+        ne_add_response_body_reader( req, ne_accept_2xx,
+                                    content_reader,
+                                    (void*) that );
+        return;
+    }
 
-        if (etag.isEmpty()) {
-            qDebug() << Q_FUNC_INFO << "No E-Tag reply by server, considering it invalid" << ne_get_response_header(req, "etag");
-            that->errorString = QLatin1String("No E-Tag received from server, check Proxy/Gateway");
-            ne_set_error(that->_propagator->_session, "No E-Tag received from server, check Proxy/Gateway");
-            ne_add_response_body_reader( req, do_not_accept,
-                                         do_not_download_content_reader,
-                                         (void*) that );
-            return;
-        } else if (!that->_expectedEtagForResume.isEmpty() && that->_expectedEtagForResume != etag) {
-            qDebug() << Q_FUNC_INFO <<  "We received a different E-Tag for resuming!"
-                     << QString::fromLatin1(that->_expectedEtagForResume.data()) << "vs"
-                     << QString::fromLatin1(etag.data());
-            that->errorString = QLatin1String("We received a different E-Tag for resuming. Retrying next time.");
-            ne_set_error(that->_propagator->_session, "We received a different E-Tag for resuming. Retrying next time.");
-            ne_add_response_body_reader( req, do_not_accept,
-                                         do_not_download_content_reader,
-                                         (void*) that );
-            return;
-        }
+    QByteArray etag = parseEtag(ne_get_response_header(req, "etag"));
+    if(etag.isEmpty())
+        etag = parseEtag(ne_get_response_header(req, "ETag"));
 
-
-        const char *enc = ne_get_response_header( req, "Content-Encoding" );
-        qDebug("Content encoding ist <%s> with status %d", enc ? enc : "empty",
-                    status ? status->code : -1 );
-
-        if( enc == QLatin1String("gzip") ) {
-            that->_decompress.reset(ne_decompress_reader( req, ne_accept_2xx,
-                                                             content_reader,     /* reader callback */
-                                                             that ));  /* userdata        */
-        } else {
-            ne_add_response_body_reader( req, ne_accept_2xx,
-                                        content_reader,
+    if (etag.isEmpty()) {
+        qDebug() << Q_FUNC_INFO << "No E-Tag reply by server, considering it invalid" << ne_get_response_header(req, "etag");
+        that->errorString = tr("No E-Tag received from server, check Proxy/Gateway");
+        ne_set_error(that->_propagator->_session, "%s", that->errorString.toUtf8().data());
+        ne_add_response_body_reader( req, do_not_accept,
+                                        do_not_download_content_reader,
                                         (void*) that );
-        }
+        return;
+    } else if (!that->_expectedEtagForResume.isEmpty() && that->_expectedEtagForResume != etag) {
+        qDebug() << Q_FUNC_INFO <<  "We received a different E-Tag for resuming!"
+                    << QString::fromLatin1(that->_expectedEtagForResume.data()) << "vs"
+                    << QString::fromLatin1(etag.data());
+        that->errorString = tr("We received a different E-Tag for resuming. Retrying next time.");
+        ne_set_error(that->_propagator->_session, "%s", that->errorString.toUtf8().data());
+        ne_add_response_body_reader( req, do_not_accept,
+                                        do_not_download_content_reader,
+                                        (void*) that );
+        return;
     }
 
-    static void notify_status_cb(void* userdata, ne_session_status status,
-                          const ne_session_status_info* info)
-    {
-        PropagateDownloadFile* that = reinterpret_cast<PropagateDownloadFile*>(userdata);
-        if (status == ne_status_recving && info->sr.total > 0) {
-            emit that->progress(Progress::Context, that->_item, info->sr.progress, info->sr.total );
-            that->limitBandwidth(info->sr.progress,  that->_propagator->_downloadLimit);
-        }
+
+    const char *enc = ne_get_response_header( req, "Content-Encoding" );
+    qDebug("Content encoding ist <%s> with status %d", enc ? enc : "empty",
+                status ? status->code : -1 );
+
+    if( enc == QLatin1String("gzip") ) {
+        that->_decompress.reset(ne_decompress_reader( req, ne_accept_2xx,
+                                                            content_reader,     /* reader callback */
+                                                            that ));  /* userdata        */
+    } else {
+        ne_add_response_body_reader( req, ne_accept_2xx,
+                                    content_reader,
+                                    (void*) that );
     }
-};
+}
+
+void PropagateDownloadFile::notify_status_cb(void* userdata, ne_session_status status,
+                        const ne_session_status_info* info)
+{
+    PropagateDownloadFile* that = reinterpret_cast<PropagateDownloadFile*>(userdata);
+    if (status == ne_status_recving && info->sr.total > 0) {
+        emit that->progress(Progress::Context, that->_item, info->sr.progress, info->sr.total );
+
+        QCoreApplication::processEvents();
+        that->limitBandwidth(info->sr.progress,  that->_propagator->_downloadLimit);
+    }
+}
 
 void PropagateDownloadFile::start()
 {
@@ -706,7 +734,7 @@ void PropagateDownloadFile::start()
         return;
     }
 
-    csync_win32_set_file_hidden(tmpFileName.toUtf8().constData(), true);
+    csync_win32_set_file_hidden(tmpFile.fileName().toUtf8().constData(), true);
 
     {
         SyncJournalDb::DownloadInfo pi;
@@ -803,7 +831,8 @@ void PropagateDownloadFile::start()
         if (dotLocation <= conflictFileName.lastIndexOf('/') + 1) {
             dotLocation = conflictFileName.size();
         }
-        conflictFileName.insert(dotLocation, "_conflict-" + QDateTime::fromTime_t(_item._modtime).toString("yyyyMMdd-hhmmss"));
+        QString timeString = Utility::qDateTimeFromTime_t(_item._modtime).toString("yyyyMMdd-hhmmss");
+        conflictFileName.insert(dotLocation, "_conflict-" + timeString);
         if (!f.rename(conflictFileName)) {
             //If the rename fails, don't replace it.
             done(SyncFileItem::NormalError, f.errorString());
@@ -811,7 +840,12 @@ void PropagateDownloadFile::start()
         }
     }
 
-    csync_win32_set_file_hidden(tmpFileName.toUtf8().constData(), false);
+    QFileInfo existingFile(fn);
+    if(existingFile.exists() && existingFile.permissions() != tmpFile.permissions()) {
+        tmpFile.setPermissions(existingFile.permissions());
+    }
+
+    csync_win32_set_file_hidden(tmpFile.fileName().toUtf8().constData(), false);
 
 #ifndef Q_OS_WIN
     bool success;
@@ -859,7 +893,6 @@ void PropagateDownloadFile::start()
     done(isConflict ? SyncFileItem::Conflict : SyncFileItem::Success);
 }
 
-DECLARE_JOB(PropagateLocalRename)
 
 void PropagateLocalRename::start()
 {
@@ -881,17 +914,19 @@ void PropagateLocalRename::start()
     SyncJournalFileRecord record(_item, _propagator->_localDir + _item._renameTarget);
     record._path = _item._renameTarget;
 
-    _propagator->_journal->setFileRecord(record);
+    if (!_item._isDirectory) { // Directory are saved at the end
+        _propagator->_journal->setFileRecord(record);
+    }
     _propagator->_journal->commit("localRename");
 
 
     done(SyncFileItem::Success);
 }
 
-DECLARE_JOB(PropagateRemoteRename)
-
 void PropagateRemoteRename::start()
 {
+    qDebug() << Q_FUNC_INFO << _item._file << "-->"<< _item._renameTarget;
+
     if (_item._file == _item._renameTarget) {
         if (!_item._isDirectory) {
             // The parents has been renamed already so there is nothing more to do.
@@ -900,7 +935,8 @@ void PropagateRemoteRename::start()
             // Note: we also update the mtime because the server do not keep the mtime when moving files
             QScopedPointer<char, QScopedPointerPodDeleter> uri2(
                 ne_path_escape((_propagator->_remoteDir + _item._renameTarget).toUtf8()));
-            updateMTimeAndETag(uri2.data(), _item._modtime);
+            if (!updateMTimeAndETag(uri2.data(), _item._modtime))
+                return;
         }
     } else if (_item._file == QLatin1String("Shared") ) {
         // Check if it is the toplevel Shared folder and do not propagate it.
@@ -918,13 +954,18 @@ void PropagateRemoteRename::start()
         qDebug() << "MOVE on Server: " << uri1.data() << "->" << uri2.data();
 
         int rc = ne_move(_propagator->_session, 1, uri1.data(), uri2.data());
+
+        if( checkForProblemsWithShared(tr("The file was renamed but is part of a read only share. The original file was restored."))) {
+            return;
+        }
+
         if (updateErrorFromSession(rc)) {
             return;
         }
 
-        updateMTimeAndETag(uri2.data(), _item._modtime);
+        if (!updateMTimeAndETag(uri2.data(), _item._modtime))
+            return;
         emit progress(Progress::EndRename, _item, _item._size, _item._size);
-
     }
 
     _propagator->_journal->deleteFileRecord(_item._originalFile);
@@ -1052,7 +1093,7 @@ void OwncloudPropagator::start(const SyncFileItemVector& _syncedItems)
             continue;
         }
 
-        while (!item._file.startsWith(directories.top().first)) {
+        while (!item.destination().startsWith(directories.top().first)) {
             directories.pop();
         }
 
@@ -1066,7 +1107,7 @@ void OwncloudPropagator::start(const SyncFileItemVector& _syncedItems)
             } else {
                 directories.top().second->append(dir);
             }
-            directories.push(qMakePair(item._file + "/" , dir));
+            directories.push(qMakePair(item.destination() + "/" , dir));
         } else if (PropagateItemJob* current = createJob(item)) {
             directories.top().second->append(current);
         }
@@ -1084,6 +1125,26 @@ void OwncloudPropagator::start(const SyncFileItemVector& _syncedItems)
     _rootJob->start();
 }
 
+void OwncloudPropagator::overallTransmissionSizeChanged(qint64 change)
+{
+    emit progressChanged(change);
+}
+
+bool OwncloudPropagator::isInSharedDirectory(const QString& file)
+{
+    bool re = false;
+    if( _remoteDir.contains("remote.php/webdav/Shared") ) {
+        // The Shared directory is synced as its own sync connection
+        re = true;
+    } else {
+        if( file.startsWith("Shared/") || file == "Shared" )  {
+            // The whole ownCloud is synced and Shared is always a top dir
+            re = true;
+        }
+    }
+    return re;
+}
+
 void PropagateDirectory::start()
 {
     _current = -1;
@@ -1097,7 +1158,7 @@ void PropagateDirectory::start()
 
 void PropagateDirectory::proceedNext(SyncFileItem::Status status)
 {
-    if (status == SyncFileItem::FatalError) {
+    if (status == SyncFileItem::FatalError || (_current == -1 && status != SyncFileItem::Success)) {
         emit finished(status);
         return;
     } else if (status == SyncFileItem::NormalError || status == SyncFileItem::SoftError) {
@@ -1114,7 +1175,7 @@ void PropagateDirectory::proceedNext(SyncFileItem::Status status)
                 _item._file = _item._renameTarget;
             }
 
-            if (_item._should_update_etag) {
+            if (_item._should_update_etag && _item._instruction != CSYNC_INSTRUCTION_REMOVE) {
                 SyncJournalFileRecord record(_item,  _propagator->_localDir + _item._file);
                 _propagator->_journal->setFileRecord(record);
             }
@@ -1123,4 +1184,5 @@ void PropagateDirectory::proceedNext(SyncFileItem::Status status)
     }
 }
 
+
 }
